<?php
ob_start();
class Faq_List_Table extends WP_List_Table{
    private $plugin_name;
    public $title_length;
    /** Class constructor */
    public function __construct($plugin_name) {
        $this->plugin_name = $plugin_name;
        $this->title_length = Faq_Builder_Ays_Admin::get_listtables_title_length('faq_title');

        parent::__construct( array(
            'singular' => __( 'F.A.Q', $this->plugin_name ), //singular name of the listed records
            'plural'   => __( 'F.A.Qs', $this->plugin_name ), //plural name of the listed records
            'ajax'     => false //does this table support ajax?
        ) );
        add_action( 'admin_notices', array( $this, 'faq_notices' ) );

    }

    /**
     * Retrieve customers data from the database
     *
     * @param int $per_page
     * @param int $page_number
     *
     * @return mixed
     */
    public function get_faqs( $per_page = 20, $page_number = 1, $search = '' ) {

        global $wpdb;

        $sql = "SELECT * FROM {$wpdb->prefix}ays_faqs";

        $where = array();

        if( $search != '' ){
            $where[] = $search;
        }

        if( ! empty($where) ){
            $sql .= " WHERE " . implode( " AND ", $where );
        }


        if ( ! empty( $_REQUEST['orderby'] ) ) {

            $order_by  = ( isset( $_REQUEST['orderby'] ) && sanitize_text_field( $_REQUEST['orderby'] ) != '' ) ? sanitize_text_field( $_REQUEST['orderby'] ) : 'id';
            $order_by .= ( ! empty( $_REQUEST['order'] ) && strtolower( $_REQUEST['order'] ) == 'asc' ) ? ' ASC' : ' DESC';

            $sql_orderby = sanitize_sql_orderby($order_by);

            if ( $sql_orderby ) {
                $sql .= ' ORDER BY ' . $sql_orderby;
            } else {
                $sql .= ' ORDER BY id DESC';
            }
        }else{
            $sql .= ' ORDER BY id DESC';
        }

        $sql .= " LIMIT $per_page";
        $sql .= ' OFFSET ' . ( $page_number - 1 ) * $per_page;


        $result = $wpdb->get_results( $sql, 'ARRAY_A' );

        return $result;
    }

    public function get_faq_by_id( $id ){
        global $wpdb;

        $sql = "SELECT * FROM {$wpdb->prefix}ays_faqs WHERE id=" . absint( sanitize_text_field( $id ) );

        $result = $wpdb->get_row($sql, 'ARRAY_A');

        $json = ( isset($result['style']) && $result['style'] != '' ) ? $result['style'] : '';
        if($json != ''){
            $result['style'] = json_decode($json, TRUE);
        }
        
        $json = ( isset($result['options']) && $result['options'] != '' ) ? $result['options'] : '';
        if( $json != '' ){
            $result['options'] = json_decode($json, TRUE);
        }

        return $result;
    }

    public function add_or_edit_faqs($data){
        global $wpdb;
        $faq_table = $wpdb->prefix . 'ays_faqs';
        if( isset($data["faq_action"]) && wp_verify_nonce( $data["faq_action"],"faq_action" ) ) {
                $id = ( $data["id"] != NULL ) ? absint( intval( $data["id"] ) ) : null;
                $title = (isset($data["ays_faq_title"]) && $data["ays_faq_title"] != '') ? wp_unslash(sanitize_text_field( $data["ays_faq_title"] )) : 'Default title';
                $description = (isset($data["ays_faq_description"]) &&  $data["ays_faq_description"] != '') ? stripslashes( $data["ays_faq_description"] ) : 'Default description';

                $question = (isset($data['ays_faq_question']) && !empty($data['ays_faq_question'])) ? implode( "***", $data['ays_faq_question'] ) : array();
                $answer   = (isset($data['ays_faq_answer']) && !empty($data['ays_faq_answer'])) ? implode( "***", $data['ays_faq_answer'] ) : array();

                $style_theme          = (isset( $data["ays_faq_theme"]) &&  $data["ays_faq_theme"] != '') ? wp_unslash(sanitize_text_field( $data["ays_faq_theme"] )) :  'default_arrow';
                $style_iconDirection  = (isset($data["ays_faq_iconDirection"]) && $data["ays_faq_iconDirection"] != '') ?sanitize_text_field( $data["ays_faq_iconDirection"] ) : 'after';
                $style_bgColor        = (isset($data["bgcolor"]) && $data["bgcolor"] != '') ? sanitize_text_field( $data["bgcolor"] ) : '#f0f0f0';
                $bgColorQuest         = ( isset($data["bgcolorquest"]) && $data["bgcolorquest"] != '') ? sanitize_text_field( $data["bgcolorquest"] ) : '#DFDDDD';
                $bgColorAnsw          = (isset($data["bgcoloransw"]) && $data["bgcoloransw"] != '') ? sanitize_text_field( $data["bgcoloransw"] ) : '#DFDDDD';
                $titleBgColor         = (isset($data["titleBgColor"]) && $data["titleBgColor"] != '') ? sanitize_text_field( $data["titleBgColor"] ) : '#f9f9f9';
                $textColor            = (isset($data["textcolor"]) && $data["textcolor"] != '') ? sanitize_text_field( $data["textcolor"] ) : '#000000';
                $questtextColor       = (isset($data["questtextColor"]) && $data["questtextColor"] != '') ? sanitize_text_field( $data["questtextColor"] ) : '#000000';
                $answertextColor      = (isset($data["answertextColor"]) && $data["answertextColor"] != '') ? sanitize_text_field( $data["answertextColor"] ) : '#000000';
                $faqBorderRadius      = (isset($data["faqBorderRadius"]) && $data["faqBorderRadius"] != '') ? absint(intval( $data["faqBorderRadius"] )) : '0';
                $questBorderRadius    = (isset($data["questBorderRadius"]) && $data["questBorderRadius"] != '') ? absint(intval( $data["questBorderRadius"] )) : '0';
                $questSpaceBetween    = (isset($data["questSpaceBetween"]) && $data["questSpaceBetween"] != '') ? absint(intval( $data["questSpaceBetween"] )) : '0';
                $ansHeight            = (isset($data["ansHeight"]) && $data["ansHeight"] != '') ? absint(intval( $data["ansHeight"] )) : 'auto';
                // $ansPadding           = sanitize_text_field( $data["ansPadding"] );
                $faqWidth             = (isset($data["faqWidth"]) && $data["faqWidth"] != '') ? absint(intval( $data["faqWidth"] )) : '';               
                $ays_faq_border_width = (isset($data["ays_faq_border_width"]) && $data["ays_faq_border_width"] != '') ? absint(intval( $data["ays_faq_border_width"] )) : '0';
                $ays_faq_border_style = (isset($data["ays_faq_border_style"]) && $data["ays_faq_border_style"] != '') ? sanitize_text_field( $data["ays_faq_border_style"] ) : 'solid';
                $ays_faq_border_color =  (isset($data["bordercolor"]) && $data["bordercolor"] != '') ? sanitize_text_field($data["bordercolor"]) : '#ffffff';
                $collapseStyle        = (isset($data["collapseStyle"]) && $data["collapseStyle"] != '') ? sanitize_text_field( $data["collapseStyle"] ) : '';
                $sendQuestionToAdmin  = isset($data["sendQuestionToAdmin"]) ? sanitize_text_field( $data["sendQuestionToAdmin"] ) : array(); //array
                $ays_faq_Icon         = (isset($data["ays_faq_Icon"]) && $data["ays_faq_Icon"] != '') ? sanitize_text_field( $data["ays_faq_Icon"] ) : '';
                $customClass          = (isset($data["ays_custom_class"]) && $data["ays_custom_class"] != '') ? sanitize_text_field( $data["ays_custom_class"] ) : '';
                $customCss            = (isset($data["customCss"]) && $data["customCss"] != '') ? stripslashes( esc_attr($data["customCss"]) ) : '';

                $ays_faq_quest_font_size = (isset($data['ays_faq_question_icon_font_size']) && $data['ays_faq_question_icon_font_size'] != '') ? absint($data['ays_faq_question_icon_font_size']) : '18';
                
                $ays_faq_show_title = (isset($data['ays_faq_show_title']) && $data['ays_faq_show_title'] == 'on') ? 'on' : 'off';
                $ays_faq_show_desc = (isset($data['ays_faq_show_description']) && $data['ays_faq_show_description'] == 'on') ? 'on' : 'off';

                $faq_answers_left_right_padding = (isset($data['ays_faq_answers_left_right_padding']) && $data['ays_faq_answers_left_right_padding'] != '') ? absint($data['ays_faq_answers_left_right_padding']) : '11';
                $faq_answers_top_bottom_padding = (isset($data['ays_faq_answers_top_bottom_padding']) && $data['ays_faq_answers_top_bottom_padding'] != '') ? absint($data['ays_faq_answers_top_bottom_padding']) : '11';

                $faq_questions_left_right_padding = (isset($data['ays_faq_questions_left_right_padding']) && $data['ays_faq_questions_left_right_padding'] != '') ? absint($data['ays_faq_questions_left_right_padding']) : '11';
                $faq_questions_top_bottom_padding = (isset($data['ays_faq_questions_top_bottom_padding']) && $data['ays_faq_questions_top_bottom_padding'] != '') ? absint($data['ays_faq_questions_top_bottom_padding']) : '11';

                // $ays_faq_quest_alignment = (isset($data['ays_faq_quest_alignment']) && $data['ays_faq_quest_alignment'] != '') ? sanitize_text_field($data['ays_faq_quest_alignment']) : 'left';
               
                $ays_faq_ans_font_size = (isset($data['ays_faq_answer_font_size']) && $data['ays_faq_answer_font_size'] != '') ? absint($data['ays_faq_answer_font_size']) : '16';

                //Title font size
                $ays_faq_title_font_size = (isset($data['ays_faq_title_font_size']) && $data['ays_faq_title_font_size'] != '' ) ? absint($data['ays_faq_title_font_size']) : 21;

                $ays_faq_title_transformation = (isset($data["ays_faq_title_transformation"]) && $data["ays_faq_title_transformation"] != '') ? sanitize_text_field( $data["ays_faq_title_transformation"] ) : 'none';

                $ays_faq_quest_height = (isset($data['ays_faq_quest_height']) && $data['ays_faq_quest_height'] != '') ? absint($data['ays_faq_quest_height']) : 'auto';

                $ays_faq_hover_active_text_color = (isset($data['ays_faq_hover_active_text_color']) && $data['ays_faq_hover_active_text_color'] != '') ? sanitize_text_field($data['ays_faq_hover_active_text_color']) : '#dfdddd';
                
                $ays_faq_hover_active_bg_color = (isset($data['ays_faq_hover_active_bg_color']) && $data['ays_faq_hover_active_bg_color'] != '') ? sanitize_text_field($data['ays_faq_hover_active_bg_color']) : '#000000';

                $ays_faq_display_all = (isset($data['ays_faq_display_all']) && $data['ays_faq_display_all'] != '') ? sanitize_text_field($data['ays_faq_display_all']) : 'no';

                //Question border options start

                //Border checkbox
                $ays_faq_questions_border = (isset($data['ays_faq_questions_border']) && $data['ays_faq_questions_border'] == 'on') ? 'on' : 'off';

                //Border side
                $ays_faq_quest_border_side = (isset($data['ays_faq_border_side']) && $data['ays_faq_border_side'] != '') ? sanitize_text_field($data['ays_faq_border_side']) : 'none';

                //Border width
                $ays_faq_quest_border_width = (isset($data['ays_faq_questions_border_width']) && $data['ays_faq_questions_border_width'] != '') ? absint($data['ays_faq_questions_border_width']) : '0';

                //Border style
                $ays_faq_quest_border_style = (isset($data['ays_faq_questions_border_style']) && $data['ays_faq_questions_border_style'] != '') ? sanitize_text_field($data['ays_faq_questions_border_style']) : 'solid';

                //Border color
                $ays_faq_quest_border_color = (isset($data['ays_questions_border_color']) && $data['ays_questions_border_color'] != '') ? sanitize_text_field($data['ays_questions_border_color']) : '#000000';
                
                //Question border options end
                
                $ays_faq_show_question_numbering = (isset($data['ays_faq_show_question_numbering']) && $data['ays_faq_show_question_numbering'] != '') ? sanitize_text_field($data['ays_faq_show_question_numbering']) : 'none';

                $faqMobileWidth       = (isset($data["ays_faq_mobile_width"]) && $data["ays_faq_mobile_width"] != '') ? absint(intval( $data["ays_faq_mobile_width"] )) : '';

                // With editor
                $with_editor = (isset($data['ays_faq_question_open_editor']) && !empty($data['ays_faq_question_open_editor'])) ? implode(',',$data['ays_faq_question_open_editor']) : array();

                //Description font size
                $ays_faq_desc_font_size = (isset($data['ays_faq_desc_font_size']) && $data['ays_faq_desc_font_size'] != '' ) ? absint($data['ays_faq_desc_font_size']) : 16;

                $faq_create_date = isset( $data['ays_faq_create_date'] ) ? sanitize_text_field( $data['ays_faq_create_date'] ) : '0000-00-00 00:00:00';
                
                $new_user_questions = "";
                if(isset( $data["user_quest"] ) && $data["user_quest"]=="true" ) { //aray
                    $new_user_questions .= implode( "****", $data['ays_faq_user_question'] );
                }
                $style = json_encode(array(
                    'style_theme'                   => $style_theme,
                    'icon_direction'                => $style_iconDirection,
                    'bgColor'                       => $style_bgColor,
                    'bgColorQuest'                  => $bgColorQuest,
                    'bgColorAnsw'                   => $bgColorAnsw,
                    'titleBgColor'                  => $titleBgColor,
                    'textColor'                     => $textColor,
                    'questtextColor'                => $questtextColor,
                    'answertextColor'               => $answertextColor,
                    'questBorderRadius'             => $questBorderRadius,
                    'faqBorderRadius'               => $faqBorderRadius,
                    'questSpaceBetween'             => $questSpaceBetween,
                    'ansHeight'                     => $ansHeight,
                    'faqWidth'                      => $faqWidth,                   
                    'ays_faq_border_width'          => $ays_faq_border_width,
                    'ays_faq_border_style'          => $ays_faq_border_style,
                    'ays_faq_border_color'          => $ays_faq_border_color,
                    'ays_faq_Icon'                  => $ays_faq_Icon,
                    // 'ansPadding'                 => $ansPadding,
                    'customClass'                   => $customClass,
                    'customCss'                     => $customCss,
                    'quest_font_size'               => $ays_faq_quest_font_size,
                    'ays_faq_show_title'            => $ays_faq_show_title,
                    'ays_faq_show_desc'             => $ays_faq_show_desc,
                    'answers_left_right_padding'    => $faq_answers_left_right_padding,
                    'answers_top_bottom_padding'    => $faq_answers_top_bottom_padding,
                    'questions_left_right_padding'  => $faq_questions_left_right_padding,
                    'questions_top_bottom_padding'  => $faq_questions_top_bottom_padding,
                    // 'quest_alignment'               => $ays_faq_quest_alignment,
                    'answer_font_size'              => $ays_faq_ans_font_size,
                    'question_height'               => $ays_faq_quest_height,
                    'hover_active_text_color'       => $ays_faq_hover_active_text_color,
                    'hover_active_bg_color'         => $ays_faq_hover_active_bg_color,
                    'questions_border_side'         => $ays_faq_quest_border_side,
                    'questions_border_width'        => $ays_faq_quest_border_width,
                    'questions_border_style'        => $ays_faq_quest_border_style,
                    'questions_border_color'        => $ays_faq_quest_border_color,
                    'enable_questions_border'       => $ays_faq_questions_border,
                    'title_font_size'               => $ays_faq_title_font_size,
                    'title_transformation'          => $ays_faq_title_transformation,
                    'faqMobileWidth'                => $faqMobileWidth,
                    'desc_font_size'                => $ays_faq_desc_font_size,
                ));
         
                $options = json_encode(array(
                    'collapseStyle'           => $collapseStyle,
                    'sendQuestionToAdmin'     => $sendQuestionToAdmin, //aray
                    'display_all'             => $ays_faq_display_all,
                    'show_question_numbering' => $ays_faq_show_question_numbering,
                    'with_editor'             => $with_editor,
                    'create_date'             => $faq_create_date,
                ));

                $submit_type = (isset($data['submit_type'])) ?  $data['submit_type'] : '';
                
                if ( $id == null ) {
                    $faq_result = $wpdb->insert(
                        $faq_table,
                        array(
                            'title' => $title,
                            'description' => $description,
                            'questions' => $question,
                            'answers' => $answer,
                            'style' => $style,
                            'options' => $options
                        ),
                        array(
                            '%s', // title
                            '%s', // description
                            '%s', // questions
                            '%s', // answers
                            '%s', // style
                            '%s', // options
                        )
                    );
                    $message = 'created';
                }else{
                    $faq_result = $wpdb->update(
                        $faq_table,
                        array(
                            "title" => $title,
                            "description" => $description,
                            'questions' => $question,
                            'answers' => $answer,
                            'style' => $style,
                            'options' => $options,
                            'user_question' => $new_user_questions //aray
                        ),
                        array( "id" => $id ),
                        array(
                            '%s', // title
                            '%s', // description
                            '%s', // questions
                            '%s', // answers
                            '%s', // style
                            '%s', // options
                            '%s', // user_question
                        )
                    );
                    $message = "updated";
                }
                $ays_faq_tab = isset($data['ays_faq_tab']) ? $data['ays_faq_tab'] : 'tab1';
                if ($faq_result >= 0) {
                    if($submit_type == ''){
                        $url = esc_url_raw( remove_query_arg(["action", "faq"]  ) ) . "&status=" . $message . "&type=success";
                        wp_redirect( $url );
                        exit();
                    }else{
                        if($id == null){
                            $url = esc_url_raw( add_query_arg( array(
                                "action"  => "edit",
                                "faq"  => $wpdb->insert_id,
                                "status" => $message,
                                "ays_faq_tab" => $ays_faq_tab
                            ) ) );
                        }else{
                            $url = esc_url_raw( remove_query_arg(false) ) . '&ays_faq_tab='.$ays_faq_tab.'&status=' . $message;'&status=' . $message;
                        }
                        wp_redirect( $url );
                        exit();
                    }
                }
        }
    }

    public function duplicate_faq($id){
        global $wpdb;
        $faq_table = $wpdb->prefix . 'ays_faqs';

        $faqDup = $this->get_faq_by_id($id);

        $options = json_decode($faqDup['options'], true);
        
        $options['create_date'] =  current_time( 'mysql' );

        $faq_result = $wpdb->insert(
            $faq_table,
            array(
                'title' => "Copy - ".$faqDup['title'],
                'description' => $faqDup['description'],
                'questions' => $faqDup['questions'],
                'answers' => $faqDup['answers'],
                'style' => json_encode($faqDup['style']),
                'options' => json_encode($faqDup['options'])
            ),
            array(
                '%s', // title
                '%s', // description
                '%s', // questions
                '%s', // answers
                '%s', // style
                '%s', // options
            )
        );
        $message = 'duplicated';
        if ($faq_result >= 0) {
            $url = esc_url_raw( remove_query_arg(array("action", "faq")  ) ) . "&status=" . $message . "&type=success";
            wp_redirect( $url );
        }
    }

    /**
     * Delete a customer record.
     *
     * @param int $id customer ID
     */
    public static function delete_faqs( $id ) {
        global $wpdb;
        $wpdb->delete(
            "{$wpdb->prefix}ays_faqs",
            array( 'id' => $id ),
            array( '%d' )
        );
    }


    /**
     * Returns the count of records in the database.
     *
     * @return null|string
     */
    public static function record_count() {
        global $wpdb;

        $filter = array();

        $sql = "SELECT COUNT(*) FROM {$wpdb->prefix}ays_faqs";

        $search = ( isset( $_REQUEST['s'] ) ) ? esc_sql( sanitize_text_field( $_REQUEST['s'] ) ) : false;
        if( $search ){
            $filter[] = sprintf(" title LIKE '%%%s%%' ", esc_sql( $wpdb->esc_like( $search ) ) );
        }
        
        if(count($filter) !== 0){
            $sql .= " WHERE ".implode(" AND ", $filter);
        }

        return $wpdb->get_var( $sql );
    }

    /** Text displayed when no customer data is available */
    public function no_items() {
        echo __( 'There are no faqs yet.', $this->plugin_name );
    }


    /**
     * Render a column when no column specific method exist.
     *
     * @param array $item
     * @param string $column_name
     *
     * @return mixed
     */
    public function column_default( $item, $column_name ) {
        switch ( $column_name ) {
            case 'title':
        //    case 'description':
                return stripslashes($item[ $column_name ]);
                break;
            case 'shortcode':
            case 'questions_count':
            case 'create_date':
            case 'id':
                return $item[ $column_name ];
                break;
            default:
                return print_r( $item, true ); //Show the whole array for troubleshooting purposes
        }
    }

    /**
     * Render the bulk edit checkbox
     *
     * @param array $item
     *
     * @return string
     */
    function column_cb( $item ) {
        return sprintf(
            '<input type="checkbox" name="bulk-delete[]" value="%s" />', $item['id']
        );
    }


    /**
     * Method for name column
     *
     * @param array $item an array of DB data
     *
     * @return string
     */
    function column_title( $item ) {
        $delete_nonce = wp_create_nonce( $this->plugin_name . '-delete-faq' );

        $faq_title = esc_attr(stripcslashes($item['title']));
        $q = esc_attr($faq_title);
        $faq_title_length = intval( $this->title_length );

        $restitle = Faq_Builder_Ays_Admin::ays_restriction_string("word", $faq_title, $faq_title_length);

        $title = sprintf( '<a href="?page=%s&action=%s&faq=%d title="%s">%s</a>', esc_attr( $_REQUEST['page'] ), 'edit', absint( $item['id'] ), $q, $restitle );
        
        $actions = array(
            'edit'      => sprintf( '<a href="?page=%s&action=%s&faq=%d">'. __('Edit', $this->plugin_name) .'</a>', esc_attr( $_REQUEST['page'] ), 'edit', absint( $item['id'] ) ),
            'duplicate' => sprintf( '<a href="?page=%s&action=%s&faq=%d">'. __('Duplicate', $this->plugin_name) .'</a>', esc_attr( $_REQUEST['page'] ), 'duplicate', absint( $item['id'] ) ),
            'delete'    => sprintf( '<a href="?page=%s&action=%s&faq=%s&_wpnonce=%s">'. __('Delete', $this->plugin_name) .'</a>', esc_attr( $_REQUEST['page'] ), 'delete', absint( $item['id'] ), $delete_nonce )
        );

        return $title . $this->row_actions( $actions );
    }

    function column_shortcode( $item ) {
        return sprintf('<input type="text" onClick="this.setSelectionRange(0, this.value.length)" readonly value="[ays_faq id=%s]" style="width: 100%%; max-width:100%%;" />', $item["id"]);
    }

    function column_questions_count ( $item ) {
        global $wpdb;
        $sql = "SELECT `questions` FROM {$wpdb->prefix}ays_faqs WHERE id=" . absint( sanitize_text_field( $item["id"] ) );
        $result = $wpdb->get_var( $sql );
        if ($result != NULL) {
            $questions_arr = explode('***', $result);
            $count = count($questions_arr);
        } else {
            $count = 0;
        }

        return $count;
    }

    function column_create_date ( $item ) {
        $options = json_decode($item['options'], true);
        $date = (isset( $options['create_date'] ) && $options['create_date'] != '') ? $options['create_date'] : "0000-00-00 00:00:00";
        $text = "";
        if ( Faq_Builder_Ays_Admin::validateDate( $date ) ) {
            $text .= "<p><b>Date:</b> ".$date."</p>";
        }
        return $text;
    }


    /**
     *  Associative array of columns
     *
     * @return array
     */
    function get_columns() {
        $columns = array(
            'cb'                => '<input type="checkbox" />',
            'title'             => __( 'Title', $this->plugin_name ),
        //    'description'       => __( 'Description', $this->plugin_name ),
            'shortcode'         => __( 'Shortcode', $this->plugin_name ),
            'questions_count'   => __( 'Questions count', $this->plugin_name ),
            'create_date'       => __( 'Created', $this->plugin_name ),
            'id'                => __( 'ID', $this->plugin_name ),
        );

        return $columns;
    }


    /**
     * Columns to make sortable.
     *
     * @return array
     */
    public function get_sortable_columns() {
        $sortable_columns = array(
            'title'         => array( 'title', true ),
            'id'            => array( 'id', true ),
        );

        return $sortable_columns;
    }

    /**
     * Returns an associative array containing the bulk action
     *
     * @return array
     */
    public function get_bulk_actions() {
        $actions = array(
            'bulk-delete' => __('Delete', $this->plugin_name)
        );

        return $actions;
    }


    /**
     * Handles data query and filter, sorting, and pagination.
     */
    public function prepare_items() {
        global $wpdb;
        $this->_column_headers = $this->get_column_info();

        /** Process bulk action */
        $this->process_bulk_action();

        $per_page     = $this->get_items_per_page( 'faqs_per_page', 20 );
        $current_page = $this->get_pagenum();
        $total_items  = self::record_count();

        $this->set_pagination_args( array(
            'total_items' => $total_items, //WE have to calculate the total number of items
            'per_page'    => $per_page //WE have to determine how many items to show on a page
        ) );

        $search = ( isset( $_REQUEST['s'] ) ) ? esc_sql( sanitize_text_field( $_REQUEST['s'] ) ) : false;

        $do_search = ( $search ) ? sprintf(" title LIKE '%%%s%%' ", esc_sql( $wpdb->esc_like( $search ) ) ) : '';


        $this->items = $this->get_faqs( $per_page, $current_page , $do_search );
    }

    public function process_bulk_action() {
        //Detect when a bulk action is being triggered...
        $message = 'deleted';
        if ( 'delete' === $this->current_action() ) {

            // In our file that handles the request, verify the nonce.
            $nonce = esc_attr( $_REQUEST['_wpnonce'] );

            if ( ! wp_verify_nonce( $nonce, $this->plugin_name . '-delete-faq' ) ) {
                die( 'Go get a life script kiddies' );
            }
            else {
                self::delete_faqs( absint( $_GET['faq'] ) );

                // esc_url_raw() is used to prevent converting ampersand in url to "#038;"
                // add_query_arg() return the current url

                $url = esc_url_raw( remove_query_arg(array('action', 'faq', '_wpnonce')  ) ) . '&status=' . $message;
                wp_redirect( $url );
            }

        }

        // If the delete bulk action is triggered
        if ( ( isset( $_POST['action'] ) && $_POST['action'] == 'bulk-delete' )
            || ( isset( $_POST['action2'] ) && $_POST['action2'] == 'bulk-delete' )
        ) {

            $delete_ids = ( isset( $_POST['bulk-delete'] ) && ! empty( $_POST['bulk-delete'] ) ) ? esc_sql( $_POST['bulk-delete'] ) : array();

            // loop over the array of record IDs and delete them
            foreach ( $delete_ids as $id ) {
                self::delete_faqs( $id );

            }

            // esc_url_raw() is used to prevent converting ampersand in url to "#038;"
            // add_query_arg() return the current url

            $url = esc_url_raw( remove_query_arg(array('action', 'faq', '_wpnonce')  ) ) . '&status=' . $message;
            wp_redirect( $url );
        }
    }

    public function faq_notices(){
        $status = (isset($_REQUEST['status'])) ? sanitize_text_field( $_REQUEST['status'] ) : '';

        if ( empty( $status ) )
            return;

        if ( 'created' == $status )
            $updated_message = esc_html( __( 'F.A.Q created.', $this->plugin_name ) );
        elseif ( 'updated' == $status )
            $updated_message = esc_html( __( 'F.A.Q saved.', $this->plugin_name ) );
        elseif ( 'duplicated' == $status )
            $updated_message = esc_html( __( 'F.A.Q duplicated.', $this->plugin_name ) );
        elseif ( 'deleted' == $status )
            $updated_message = esc_html( __( 'F.A.Q deleted.', $this->plugin_name ) );

        if ( empty( $updated_message ) )
            return;

        ?>
        <div class="notice notice-success is-dismissible">
            <p> <?php echo $updated_message; ?> </p>
        </div>
        <?php
    }
}